from .barcodes import Barcode39, Barcode128, BarcodeAnsi
from .downloads import DownloadGraphic
from .elements import *
from .graphics import GraphicBox, GraphicCircle, GraphicDiagonalLine, GraphicEllipse, GraphicHorizontalLine, GraphicSymbol, GraphicVerticalLine
from .models import *
from .rendering import *
from .shared import *


class Engine:
    def __init__(self, elements):
        self.elements = elements

    def render(self, options):
        zpl = ''
        if options.start_end_tags:
            zpl = zpl + "^XA"

        for element in self.elements:
            zpl = zpl + (element.render(options))

        if options.start_end_tags:
            zpl = zpl + "^XZ"

        return zpl

    def to_zpl_string(self, options):
        list_of_elements = [self.elements]
        self.elements = list_of_elements
        return self.render(options)



from zpllibrary.models import NewLineConversion
from zpllibrary.shared import PositionedElement


class TextField(PositionedElement):
    def __init__(self, text, x, y, font, use_hexadecimal_indicator, reverse_print, new_line_conversion):
        super().__init__(x, y)
        self.text = text
        self.font = font
        self.hex_indicator = use_hexadecimal_indicator
        self.reverse = reverse_print
        self.new_line_conversion = new_line_conversion

    def render(self, options):
        text = self.font.render(options)
        text = text + self.origin.render(options)
        text = text + self.render_field_data()
        return text

    def render_field_data(self):
        text = "^FH" if self.hex_indicator else ""
        text = text + "^FR" if self.reverse else text + ""
        text = text + '^FD'

        for char in self.text:
            text = text + self.sanitise(char)

        text = text + '^FS'
        return text


class FieldBlock(TextField):
    def __init__(self, text, x, y, width, font, max_line_count=1, line_space=0, text_justification="L",
                 hanging_indent=0, use_hexadecimal_indicator=True, reverse_print=False,
                 new_line_conversion=NewLineConversion.NewLine):
        super().__init__(text, x, y, font, use_hexadecimal_indicator, reverse_print, new_line_conversion)
        self.text_justification = text_justification
        self.width = width
        self.max_line_count = max_line_count
        self.line_space = line_space
        self.hanging_indent = hanging_indent

    def render(self, options):
        text = self.font.render(options)
        text = text + self.origin.render(options)

        text = text + "^FB" + str(options.scale(self.width)) + "," + str(self.max_line_count) + "," + str(
            options.scale(self.line_space)) + "," + self.text_justification + "," + str(
            options.scale(self.hanging_indent))

        text = text + self.render_field_data()
        return text


class Font(object):
    def __init__(self, font_name="0", orientation="N", width=30, height=30):
        self.font_name = font_name
        self.orientation = orientation
        self.width = width
        self.height = height

    def render(self, options):
        return "^A" + self.font_name + self.orientation + "," + str(options.scale(self.height)) + "," + str(
            options.scale(self.width))


class QrCode(PositionedElement):
    def __init__(self, content, x, y, model=2, magnification_factor=2, error_collection="Q", mask_value=7):
        super().__init__(x, y)
        self.content = content
        self.model = model
        self.magnification_factor = magnification_factor
        self.error_collection = error_collection
        self.mask_value = mask_value

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^BQN," + str(self.model) + "," + str(
            options.scale(self.magnification_factor)) + "," + self.error_collection + "," + str(self.mask_value)
        text = text + "^FD" + self.error_collection + "M," + self.content + "^FS"
        return text


class RawZpl:
    def __init__(self, raw_zpl):
        self.raw_zpl = raw_zpl

    def render(self, options):
        return self.raw_zpl


class TextBlock(TextField):
    def __init__(self, text, x, y, width, height, font, use_hexadecimal_indicator=True, reverse_print=False,
                 new_line_conversion=NewLineConversion.Space):
        super().__init__(text, x, y, font, use_hexadecimal_indicator, reverse_print, new_line_conversion)

        self.width = width
        self.height = height

    def render(self, options):
        text = self.font.render(options)
        text = text + self.origin.render(options)

        text = text + "^TB" + self.font.orientation + "," + str(options.scale(self.width)) + "," + \
               str(options.scale(self.height))

        text = text + self.render_field_data()
        return text



from zpllibrary.shared import PositionedElement


class Barcode(PositionedElement):
    def __init__(self, content, x, y, height, orientation, print_interpretation_line,
                 print_interpretation_line_above_code):
        super().__init__(x, y)
        self.content = content
        self.height = height
        self.orientation = orientation
        self.print_interpretation_line = print_interpretation_line
        self.print_interpretation_line_above_code = print_interpretation_line_above_code


class Barcode39(Barcode):
    def __init__(self, content, x, y, height=100, orientation="N", print_interpretation_line=True,
                 print_interpretation_line_above_code=False, mod_43_check_digit=False):
        super().__init__(content, x, y, height, orientation, print_interpretation_line,
                         print_interpretation_line_above_code)

        self.mod_43_check_digit = mod_43_check_digit

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^B3" + self.orientation + ","
        text = text + "Y" if self.mod_43_check_digit else text + "N"
        text = text + ","
        text = text + str(options.scale(self.height)) + ","
        text = text + "Y" if self.print_interpretation_line else text + "N"
        text = text + ","
        text = text + "Y" if self.print_interpretation_line_above_code else text + "N"
        text = text + "^FD" + self.content + "^FS"
        return text

class Barcode128(Barcode):
    def __init__(self, content, x, y, height=100, orientation="N", print_interpretation_line=True,
                 print_interpretation_line_above_code=False):
        super().__init__(content, x, y, height, orientation, print_interpretation_line,
                         print_interpretation_line_above_code)

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^BC" + self.orientation + ","
        text = text + str(options.scale(self.height)) + ","
        text = text + "Y" if self.print_interpretation_line else text + "N"
        text = text + ","
        text = text + "Y" if self.print_interpretation_line_above_code else text + "N"
        text = text + "^FD" + self.content + "^FS"
        return text

class BarcodeAnsi(Barcode):
    def __init__(self, content, x, y, height, start_char, stop_char, orientation="N", print_interpretation_line=True,
                 print_interpretation_line_above_code=False, check_digit=False):
        super().__init__(content, x, y, height, orientation, print_interpretation_line,
                         print_interpretation_line_above_code)
        self.start_char = start_char
        self.stop_char = stop_char
        self.check_digit = check_digit

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^BK" + self.orientation + ","
        text = text + "Y" if self.check_digit else text + "N"
        text = text + ","
        text = text + str(options.scale(self.height)) + ","
        text = text + "Y" if self.print_interpretation_line else text + "N"
        text = text + ","
        text = text + "Y" if self.print_interpretation_line_above_code else text + "N"
        text = text + "^FD" + self.content + "^FS"

        return text





from zpllibrary.shared import PositionedElement

class GraphicElement(PositionedElement):
    def __init__(self, x, y, line_colour="B", border_thickness=1):
        super().__init__(x, y)
        self.line_colour = line_colour
        self.border_thickness = border_thickness


class GraphicBox(GraphicElement):
    def __init__(self, x, y, width, height, line_colour="B", border_thickness=1, corner_rounding=0):
        super().__init__(x, y, line_colour, border_thickness)
        self.width = width
        self.height = height
        self.corner_rounding = corner_rounding

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^GB"
        text = text + options.scale_to_string(self.width) + ","
        text = text + options.scale_to_string(self.height) + ","
        text = text + options.scale_to_string(self.border_thickness) + ","
        text = text + self.line_colour + ","
        text = text + options.scale_to_string(self.corner_rounding)
        text = text + "^FS"
        return text

class GraphicCircle(GraphicElement):
    def __init__(self, x, y, diameter, line_colour="B", border_thickness=1):
        super().__init__(x, y, line_colour, border_thickness)
        self.diameter = diameter

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^GC"
        text = text + options.scale_to_string(self.diameter) + ","
        text = text + options.scale_to_string(self.border_thickness) + ","
        text = text + self.line_colour
        text = text + "^FS"
        return text

class GraphicDiagonalLine(GraphicBox):
    def __init__(self, x, y, width, height, leaning_diagonal_right=False, line_colour="B", border_thickness=1):
        super().__init__(x, y, width, height, line_colour, border_thickness, 0)
        self.leaning_diagonal_right = leaning_diagonal_right

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^GD"
        text = text + options.scale_to_string(self.width) + ","
        text = text + options.scale_to_string(self.height) + ","
        text = text + options.scale_to_string(self.border_thickness) + ","
        text = text + self.line_colour + ","
        text = text + "R" if self.leaning_diagonal_right else text + "L"
        text = text + "^FS"
        return text


class GraphicEllipse(GraphicBox):
    def __init__(self, x, y, width, height, line_colour="B", border_thickness=1):
        super().__init__(x, y, width, height, line_colour, border_thickness, 0)

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^GE"
        text = text + options.scale_to_string(self.width) + ","
        text = text + options.scale_to_string(self.height) + ","
        text = text + options.scale_to_string(self.border_thickness) + ","
        text = text + self.line_colour
        text = text + "^FS"
        return text


class GraphicHorizontalLine(GraphicBox):
    def __init__(self, x, y, width, line_colour="B", line_thickness=1):
        super().__init__(x, y, width, 0, line_colour, line_thickness, 0)

    def render(self, options):
        return super().render(options)


class GraphicSymbol(PositionedElement):
    def __init__(self, character, x, y, width, height, orientation="N"):
        super().__init__(x, y)
        self.character = character
        self.width = width
        self.height = height
        self.orientation = orientation

    def render(self, options):
        text = self.origin.render(options)
        text = text + "^GS" + ","
        text = text + self.orientation + ","
        text = text + options.scale_to_string(self.height) + ","
        text = text + options.scale_to_string(self.width)
        text = text + "^FD"
        text = text + self.character.value
        text = text + "^FS"
        return text

class GraphicVerticalLine(GraphicBox):
    def __init__(self, x, y, height, line_colour="B", line_thickness=1):
        super().__init__(x, y, 0, height, line_colour, line_thickness, 0)

    def render(self, options):
        return super().render(options)


import math

from PIL import ImageOps, Image

from zpllibrary.shared import PositionedElement


class DownloadGraphic(PositionedElement):
    def __init__(self, x, y, image_path="",
                 storage="R"):
        super().__init__(x, y)
        self.image = image_path
        self.storage = storage

    def render(self, options):
        im = Image.open(self.image)
        width, height = im.size
        newsize = (round(options.scale(width)), round(options.scale(height)))
        im1 = im.resize(newsize)
        bytes_per_row = math.ceil(newsize[0] / 8.0) # Width
        total_bytes = math.ceil(newsize[1] * bytes_per_row) # Height
        compression_type="A"
        data = self.convert_image(im1)
        text = self.origin.render(options)
        if compression_type == "A":
            text += "^GFA" + ","
            text += str((len(data))) + ","
            text += str(total_bytes) + ","
            text += str(bytes_per_row) + ","
            text += data

            return text

    def convert_image(self, image, compression_type='A'):

        image = ImageOps.invert(image.convert('L')).convert('1')

        if compression_type == "A":
            return image.tobytes().hex().upper()



class Origin:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def render(self, options):
        return "^FO" + str(options.scale(self.x)) + "," + str(options.scale(self.y))


class PositionedElement(object):
    def __init__(self, x, y):
        self.origin = Origin(x, y)


from enum import Enum


class NewLineConversion(Enum):
    Space = 1,
    Empty = 2,
    NewLine = 3


class Options:
    def __init__(self, include_start_end_tags=True, source_print_dpi=203, target_print_dpi=203):
        self.start_end_tags = include_start_end_tags
        self.source_print_dpi = source_print_dpi
        self.target_print_dpi = target_print_dpi
        self.scale_factor = self.target_print_dpi / self.source_print_dpi

    def scale(self, input):
        return input * self.scale_factor

    def scale_to_string(self, input):
        return str(self.scale(input))


class SpecialCharacter(Enum):
    registered_tradeMark = "A"
    copyright = "B"
    trade_mark = "C"
    underwriters_laboratories_approval = "D"
    canadian_standards_association_approval = "E"
